/* grape-methods "delete"
 *               "list-alloc"
 *               "list-free"
 *  
 * Call              : GRAPE(curve_parm, "delete")();
 *                     GRAPE(curve_parm, "list-alloc")();
 *                     GRAPE(curve_parm, "list-free")();
 * 
 * Parameters        : CURVE_PARM *curve_parm
 *             
 * Description       : "delete" deletes a Curve_Parm instance. If the
 *                     curve_parm->refcount is not zero only the refcount
 *                     is decremented otherwise the instance is deleted.
 *
 *                     "list-alloc" is just a dummy method. All data is
 *                     allocated while reading the input file.
 *
 *                     "list-free" deletes all rulers and the spline
 *                     associated to curve_parm.
 *
 * Author            : Konrad Polthier
 *
 * Copyright (c) 1992  by Universitaet Bonn
 *                        Institut fuer Angewandte Mathematik
 *                        Sonderforschungsbereich 256
 *                        D-53115 Bonn
 */

#include <grape.h>
#include "curve.h"


CURVE_PARM *curve_parm_delete()
{
  CURVE_PARM *self;

  self = (CURVE_PARM *)START_METHOD(G_INSTANCE);
  ASSURE(self, "", END_METHOD(NULL));

  if(self->refcount)
    /* still references left, only decrement refcount */
    self->refcount--;
  else {
    /* last reference, delete first instance data and then the instance */
    GRAPE(self, "list-free")();
    GRAPE(self, "^delete")();
  }

  /* always return NULL from a delete method */
  END_METHOD(NULL);
}


CURVE_PARM *curve_parm_list_alloc()
{
  CURVE_PARM *self;

  self = (CURVE_PARM *)START_METHOD(G_INSTANCE);
  ASSURE(self, "", END_METHOD(NULL));

  /*
   * Dummy method for consistency.
   * Allocation is done in g_curve_parm_get_data (cv_data.c)
   * while reading the input file.
   */

  END_METHOD(self);
}


/*
 * All curve_parm ruler depend on the global time-ruler therefore
 * we have to remove this dependency before deleting the rulers.
 * The rulers were added to the standard-manager in
 * g_curve_parm_get_data (cv_data.c) therefore we have to remove
 * rulers from the manager before deleting them.
 */

static FUNCTION_RULER *remove_ruler(FUNCTION_RULER *ruler,
				    MANAGER *mgr, RULER *rultime)
{
  if(ruler) {
    /* remove dependency from time-ruler */
    GRAPE(rultime, "remove-side-effect")(ruler);
    /* remove ruler from manager */
    GRAPE(mgr, "remove-inter")(ruler);
    /* delete the ruler and reset the pointer */
    GRAPE(ruler, "delete")();
  }

  return NULL;
}


CURVE_PARM *curve_parm_list_free()
{
  CURVE_PARM *self;
  MANAGER *mgr;
  RULER *rultime;

  self = (CURVE_PARM *)START_METHOD(G_INSTANCE);
  ASSURE(self, "", END_METHOD(NULL));

  rultime = (RULER *)GRAPE(Surface, "get-time-ruler")();
  ASSURE(rultime,
	 "curve_parm_list_free: can't get timer-ruler",
	 END_METHOD(NULL));
    
  mgr = (MANAGER *)GRAPE(Manager, "get-stdmgr")();
  ASSURE(mgr,
	 "curve_parm_list_free: can't get standard-manager",
	 END_METHOD(NULL));

  self->min_rul = remove_ruler(self->min_rul, mgr, rultime);
  self->max_rul = remove_ruler(self->max_rul, mgr, rultime);
  self->dstrbn_rul = remove_ruler(self->dstrbn_rul, mgr, rultime);

  if(self->dstrbn) {
    GRAPE(self->dstrbn, "delete")();
    self->dstrbn = NULL;
  }

  END_METHOD(self);
}
