/* grape-method "read-data"
 *
 * Call              : GRAPE(time_curve, "read-data")(path);
 *
 * Parameters        : TIME_CURVE *time_curve
 *                     char *path
 *
 * Description       : Read an input file for class Time_Curve (*.cv),
 *                     create all parameter lists and the associated
 *                     rulers and compile the coordinate functions.
 *
 * Author            : Bernd Oberknapp, Konrad Polthier
 *
 * Copyright (c) 1992  by Universitaet Bonn
 *                        Institut fuer Angewandte Mathematik
 *                        Sonderforschungsbereich 256
 *                        D-53115 Bonn
 */

#include <grape.h>
#include "curve.h"


#define MAX_NUM_OF_INTERVALS 5


void add_ruler_to_mgr(RULER *, int);
int g_open_input_file(FILE **, char *, char *, char *);
int get_frame_number(FILE *, TIME_OBJECT *);
int get_int(int *, FILE *);
int g_time_parm_get_data(FILE *, TIME_PARM *, char *, char *, double *);


int g_curve_parm_get_data(FILE *fp, CURVE_PARM *cp,
			  char *name, char *path, double *time)
{
  char file_sp[MAX_STRLEN], ruler_name[MAX_STRLEN];
  double min_minimum, max_maximum;
  double *d_dummy, *f_dummy;
  int mnoi = 5;

  /* read number of points */
  if(!get_int(&cp->num_of_points, fp)) {
    fprintf(stderr,
	    "g_curve_parm_get_data: error in reading num_of_points\n");
    return 0;
  }

  /*
   * read in minimun:
   *    compile minimum expression into a t_fun
   *    create the minimun ruler
   *    read in the associated spline file or convert t_fun to spline
   *    add ruler to the manager
   */
  ASSURE(get_t_fun(&cp->t_min, "t", fp),
	 "g_curve_parm_get_data: can't read minimun", return 0);
  sprintf(ruler_name, "%s%s_min", path, name);
  cp->min = 0.0;
  cp->min_rul = (FUNCTION_RULER *)GRAPE(Function_Ruler, "new-instance")
    (NULL, time, &cp->min, ruler_name, RULER_DOUBLE);
  if(!(cp->min_rul->spline->status & spLoaded))
    tfun2spline(cp->t_min, 0.0, 1.0, mnoi, ruler_name, cp->min_rul->spline);
  GRAPE(cp->min_rul, "set-effect")(ON);
  add_ruler_to_mgr((RULER *)cp->min_rul, 1);

  /* read in maximum - same as for minimun */
  ASSURE(get_t_fun(&cp->t_max, "t", fp),
	 "g_curve_parm_get_data: can't read maximum", return 0);
  sprintf(ruler_name, "%s%s_max", path, name);
  cp->max = 0.0;
  cp->max_rul = (FUNCTION_RULER *)GRAPE(Function_Ruler, "new-instance")
    (NULL, time, &cp->max, ruler_name, RULER_DOUBLE);
  if(!(cp->max_rul->spline->status & spLoaded))
    tfun2spline(cp->t_max, 0.0, 1.0, mnoi, ruler_name, cp->max_rul->spline);
  GRAPE(cp->max_rul, "set-effect")(ON);
  add_ruler_to_mgr((RULER *)cp->max_rul, 1);

  /* read in distribution:
   *    try to read in a distribution spline or (if no file exists)
   *    create an appropriate linear spline and
   *    bind the spline to the distribution function ruler
   */
  sprintf(ruler_name, "%s_dstrbn", name);
  if(cp->dstrbn == NULL)
    cp->dstrbn = (SPLINE1D *)GRAPE(Spline1d, "new-instance")(ruler_name);
  sprintf(file_sp, "%s%s.sp", path, ruler_name);
  if(fopen(file_sp, "r") != NULL) {
    MESSAGE(("file = ::%s:: found\n", file_sp));
    GRAPE(cp->dstrbn, "read")(ruler_name);
  } else {
    min_minimum = tfun_min(cp->t_min, 0.0, 1.0);
    max_maximum = tfun_max(cp->t_max, 0.0, 1.0);
    if(min_minimum < max_maximum)
      GRAPE(cp->dstrbn, "set-linear")
	(min_minimum, max_maximum, min_minimum, max_maximum, 3, 10);
    else
      GRAPE(cp->dstrbn, "set-linear")
	(max_maximum, min_minimum, max_maximum, min_minimum, 3, 10);
  }
  d_dummy = double_alloc(2);
  f_dummy = d_dummy + 1;
  *d_dummy = *f_dummy = 0.0;
  cp->dstrbn_rul = (FUNCTION_RULER *)
    GRAPE(Function_Ruler, "new-instance")
      (cp->dstrbn, d_dummy, f_dummy, ruler_name, RULER_DOUBLE);
  GRAPE(cp->dstrbn_rul, "set-effect")(ON);
  add_ruler_to_mgr((RULER *)cp->dstrbn_rul, 0);

  return 1;
}


TIME_CURVE *time_curve_read_data(char *path)
{
  TIME_CURVE *self;
  FILE *fp;
  int i;
  char var_list[20];

  self = (TIME_CURVE *)START_METHOD(G_INSTANCE);
  ASSURE(self, "", END_METHOD(NULL));

  MESSAGE(("time_curve-read-data: reading %s\n", self->name));

  /* open the input file with name self->name and extension cv */
  ASSURE(g_open_input_file(&fp, self->name, path, "cv"),
	 "time_curve_read_data: can't open inputfile\n",
	 END_METHOD(NULL));

  /* get task and set up the descriptor */
  ASSURE(self->task = g_read_task(fp, self->task_mask),
	 "time_curve_read_data: error in inputfile: unknown task\n",
	 END_METHOD(NULL));

  /* get frame number and maximun number of frames */
  ASSURE(get_frame_number(fp, (TIME_OBJECT *)self),
	 "time_curve_read_data: error in inputfile: unknown frame number",
	 END_METHOD(NULL));

  /* read data (parameters) associated with superclass Time_Object */
  ASSURE(g_time_parm_get_data(fp, self->time_parm,
			      self->name, path, self->time),
	 "time_curve_read_data: error in inputfile: error in time_parm",
	 END_METHOD(NULL));

  /*
   * set default-action for the parameter rulers.
   * the method "parameter-changed" triggers an instant recalculation
   * of the time-curve if a parameter is changed.
   */
  for(i = 0; i < self->time_parm->num_int_parm; i++)
    if(self->time_parm->int_rul[i]) {
      GRAPE(self->time_parm->int_rul[i], "set-instance")(self);
      GRAPE(self->time_parm->int_rul[i], "set-action")("parameter-changed");
    }
  for(i = 0; i < self->time_parm->num_double_parm; i++)
    if(self->time_parm->dbl_rul[i]) {
      GRAPE(self->time_parm->dbl_rul[i], "set-instance")(self);
      GRAPE(self->time_parm->dbl_rul[i], "set-action")("parameter-changed");
    }

  /* read curve_parm data (see above) */
  ASSURE(g_curve_parm_get_data(fp, self->curve_parm,
			       self->name, path, self->time),
	 "time_curve_read_data: error in inputfile: error in curve_parm",
	 END_METHOD(NULL));

  /* same as for the time_parm rulers */
  GRAPE(self->curve_parm->min_rul, "set-instance")(self);
  GRAPE(self->curve_parm->min_rul, "set-action")("parameter-changed");
  GRAPE(self->curve_parm->max_rul, "set-instance")(self);
  GRAPE(self->curve_parm->max_rul, "set-action")("parameter-changed");
  GRAPE(self->curve_parm->dstrbn_rul, "set-instance")(self);
  GRAPE(self->curve_parm->dstrbn_rul, "set-action")("parameter-changed");

  /*
   * create variable list according to the functions' parameter number,
   * first parameter is the point were the functions will be evaluated (x),
   * all other parameters are handled by the time_object function
   */
  var_list[0] = 'x'; var_list[1] = ' ';
  g_time_parm_get_var_list(self->time_parm, &(var_list[2]));

  /*
   * compile the functions from string representation into a "t_fun",
   * an internal representation which can be evaluated by "fn_eval"
   * (see method "compute-send").
   */
  ASSURE(get_t_fun(&self->coord_f.x, var_list, fp) &&
	 get_t_fun(&self->coord_f.y, var_list, fp) &&
	 get_t_fun(&self->coord_f.z, var_list, fp),
	 "time_curve_read_data: can't read coordinate functions",
	 { fclose(fp); END_METHOD(NULL); });

  fclose (fp);

  /* allocate geometry and curve_parm lists */
  self = (TIME_CURVE *)GRAPE(self, "list-alloc")();

  /*
   * we have changed the object's task and frame/time by
   * hand therefore we have to update is now
   */
  GRAPE(self, "set-task")(self->task);
  GRAPE(self, "set-frame")(self->frame);

  /* compute all frames */
  GRAPE(self, "compute-video-send")();

  END_METHOD(self);
}
