#include <grape.h>
#include "curve.h"


/* grape-methods "new-instance"
 *               "delete"
 *
 * Call              : GRAPE(time_curve, "new-instance")(name);
 *                     GRAPE(time_curve, "delete")();
 *
 * Parameters        : TIME_CURVE *time_curve
 *                     char *name
 *
 * Description       : "new-instance" allocates a Time_Curve instance.
 *
 *                     "delete" deletes a Time_Curve instance. If the
 *                     time_curve->refcount is not zero only the refcount
 *                     is decremented otherwise all data allocated in
 *                     "list-alloc" and "read-data" is freed and the
 *                     instance is deleted.
 *
 * Author            : Konrad Polthier
 *
 * Copyright (c) 1992  by Universitaet Bonn
 *                        Institut fuer Angewandte Mathematik
 *                        Sonderforschungsbereich 256
 *                        D-53115 Bonn
 */

TIME_CURVE *time_curve_new_instance(char *name)
{
  TIME_CURVE *self;
  CLASS *class;

  /* "new-instance" has to be a class-method */
  class = (CLASS *)START_METHOD(G_CLASS);
  ASSURE(class, "", END_METHOD(NULL));

  /* create superclass instance */
  self = (TIME_CURVE *)GRAPE(class, "^new-instance")(name);
  ASSURE(self,
	 "time_curve_new_instance: can't create time_curve instance",
	 END_METHOD(NULL));

  /* initialize curve_parm variables */
  self->curve_parm  = (CURVE_PARM *)
    GRAPE(Curve_Parm, "new-instance")("curve_parm");

  /* no coordinate functions yet, will be allocated in "read-data" */
  self->coord_f.x = NULL;
  self->coord_f.y = NULL;
  self->coord_f.z = NULL;

  END_METHOD(self);
}


TIME_CURVE *time_curve_delete()
{
  TIME_CURVE *self;

  self = (TIME_CURVE *)START_METHOD(G_INSTANCE);
  ASSURE(self, "", END_METHOD(NULL));

  if(self->refcount)
    /* still references left, only decrement refcount */
    self->refcount--;
  else {
    /* last reference, delete instance */

    /*
     * test not necessary, only for efficiency.
     * "list-free" calls "^list-free" therefore
     * for subclasses there would be multiple
     * calls of "list-free" without this test
     */
    if(self->class == Time_Curve)
      GRAPE(self, "list-free")();

    /* delete data allocated in "new-instance" */
    if(self->curve_parm)
      GRAPE(self->curve_parm, "delete")();

    /* delete instance itsself */
    GRAPE(self, "^delete")();
  }

  END_METHOD(NULL);
}


/* grape methods "list-alloc"
 *               "list-free"
 *
 * Call              : GRAPE(time_curve, "list-alloc")();
 *                     GRAPE(time_curve, "list-free")();
 *
 * Parameters        : TIME_CURVE *time_curve
 *
 * Description       : "list-alloc" allocates all memory lists for
 *                     a Time_Curve instance. Some time_curve variables
 *                     (num_of_frames, number_of_points) have to be
 *                     initialized before calling this method.
 *
 *                     "list-free" frees all memory allocated in
 *                     "list-alloc" and "read-data".
 *
 * Author            : Konrad Polthier
 *
 * Copyright (c) 1992  by Universitaet Bonn
 *                        Institut fuer Angewandte Mathematik
 *                        Sonderforschungsbereich 256
 *                        Wegelerstr. 6
 *                        D-53115 Bonn
 *
 * History           :  created: Sep 8, 92 (kp).
 *                      revised: Apr 20, 93 (bo). checks added
 */

TIME_CURVE *time_curve_list_alloc()
{
  TIME_CURVE *self;
  TIMESTEP *step;

  self = (TIME_CURVE *)START_METHOD(G_INSTANCE);
  ASSURE(self, "", END_METHOD(NULL));

  /* allocate superclass lists first */
  GRAPE(self, "^list-alloc")();

  /*
   * create a geometry (chain) with timesteps
   * which have triang1d instances as objects
   */
  self->geometry = (CHAIN *)GRAPE(Chain, "new-instance")("curve");
  self->geometry->object = (INSTANCE *)GRAPE(TimeStep, "alloc")
    (self->num_of_frames, 0.0, 1.0, "curve");
  GRAPE(self->geometry->object, "create-objects")(Triang1d, "curve");

  /*
   * allocate memory for self->curve_parm->number_of_points
   * points for each triang1d instance
   */
  step = (TIMESTEP *)self->geometry->object;
  do {
    GRAPE(step->pre_object, "list-alloc")
      (self->curve_parm->num_of_points);
    step = step->post_step;
  } while(step && step->pre_step->time < step->time);

  /*
   * nothing allocated in curve_parm-"list-alloc" at the
   * moment, all allocation is done in "read-data"
   */
  GRAPE(self->curve_parm, "list-alloc")();

  END_METHOD(self);
}


TIME_CURVE *time_curve_list_free()
{
  TIME_CURVE *self;

  self = (TIME_CURVE *) START_METHOD(G_INSTANCE);
  ASSURE(self, "", END_METHOD(NULL));

  /*
   * delete everything allocated in "list-alloc"
   * but in reverse order (first curve_parm data
   * and then superclass data)
   */
  if(self->curve_parm)
    GRAPE(self->curve_parm, "list-free")();

  GRAPE(self, "^list-free")();

  END_METHOD(self);
}
